/**
 * Fichero : data_base.h      Fecha: 04/03/2017                           
 * 
 *  TAD para representar una base de datos de
 *  tweets y twitteros
 *                                            
 */
#ifndef __DATA_BASE_H_
#define __DATA_BASE_H_

#include "twittero.h"
#include "tweet.h"
#include "const.h"
#include "list.h"

/** Estructura del TAD base de datos */
typedef struct _data_base data_base;

/**=====================================================================
 * 1ª SEMANA
 * ===================================================================*/

/**
 * Funcion para crear una nueva base de datos, que 
 * contendra los datos existentes en el directorio
 * pasado como argumento.
 *
 * Entrada:
 *    Directorio que contendra los ficheros tweets.txt y twitteros.txt
 * Salida:
 *    Una estructura data_base inicializada o NULL en caso de error
 */
data_base *new_data_base(const char *dirname);

/**
 * Funcion para liberar la memoria reservada y los recursos de la
 * base de datos.
 *
 * Entrada:
 *    Estructura data_base a liberar
 */
void destroy_data_base(data_base *db);

/**=====================================================================
 * 2ª y 3ª SEMANA
 * ===================================================================*/

/**
 * Funcion para recuperar un tweet dado su id.
 *
 * Entrada:
 *    Estructura base de datos y el id del tweet a recuperar.
 *
 * Salida:
 *    Un puntero a la pelicula o NULL si no se encuentra.
 */
tweet *find_tweet_by_id(data_base * db, long long int id);

/**
 * Funcion para recuperar un twittero dado su id.
 *
 * Entrada:
 *    Estructura base de datos y el id del twittero a recuperar.
 *
 * Salida:
 *    Un puntero al actor o NULL si no se encuentra.
 */
twittero *find_twittero_by_id(data_base * db, long long int id);

/**
 * Funcion para recuperar un tweet dado un texto a buscar
 *
 * Entrada:
 *    Estructura base de datos y parte del texto del tweet a recuperar.
 *
 * Salida:
 *    Un puntero al tweet o NULL si no se encuentra.
 */
tweet *find_tweet_by_text(data_base *db, char *text);

/**
 * Funcion para recuperar un twittero dado su alias.
 *
 * Entrada:
 *    Estructura base de datos y el alias del twittero a recuperar.
 *
 * Salida:
 *    Un puntero al twittero o NULL si no se encuentra.
 */
twittero *find_twittero_by_alias(data_base *db, char * alias);

/**
 * Funcion para recuperar una lista con todos los twitteros.
 *
 * Entrada:
 *    Estructura base de datos.
 *
 * Salida:
 *    Un puntero a la lista o NULL si no se encuentra.
 */
list *find_all_twitteros(data_base *db);

/**
 * Funcion para recuperar una lista con todos los tweets.
 *
 * Entrada:
 *    Estructura base de datos.
 *
 * Salida:
 *    Un puntero a la lista o NULL si no se encuentra.
 */
list *find_all_tweets(data_base *db);

/**
 * Funcion para recuperar una lista con todos los hashtags en aquellos
 * tweets en los que se menciona a un twittero.
 * A traves de dicha lista deben exponerse el hashtag y su número de
 * repeticiones debidamente ordenados de mayor a menor
 * [ver estructura 'hashtag_number']
 *
 * Entrada:
 *    Estructura base de datos y alias del tuiterro.
 *
 * Salida:
 *    Un puntero a la lista o NULL si no se encuentra.
 */
list *find_hashtags_per_mentioned(data_base *db, char *alias);

/**=====================================================================
 * 4ª SEMANA
 * ===================================================================*/

/**
 * Funcion para añadir un nuevo tweet en la base de datos.
 * Se comprueba que el twittero de la este
 * previamente añadido en la base de datos.
 *
 * Entrada:
 *    La base de datos donde se añade y el tweet a añadir.
 * Salida:
 *    Falso si el tweet no se pudo añadir por no cumplir alguna restriccion.
 */
BOOL data_base_add_tweet(data_base *db, tweet *t);

/**
 * Funcion para añadir un nuevo twittero en la base de datos.
 * Comprueba que el twittero no este repetido (según su alias y su 
 * nombre).
 *
 * Entrada:
 *    La base de datos donde se añade y el twittero a añadir.
 * Salida:
 *    Falso si el twittero esta repetido y no se pudo añadir.
 */
BOOL data_base_add_twittero(data_base *db, twittero *tt);

/**
 * Funcion para eliminar un tweet de la base de datos.
 * La memoria ocupada por el tweet es liberado.
 * El tweet debe estar registrada en la base de datos.
 *
 * Entrada:
 *    La base de datos y el tweet a eliminar.
 * Salida:
 *    Tras la llamada el tweet es liberado y se devuelve TRUE.
 *    Si el tweet no existe en la base de datos se devuelve FALSO.
 */
BOOL data_base_remove_tweet(data_base *db, tweet *t);

/**
 * Funcion para eliminar un twittero de la base de datos.
 * La memoria ocupada por el twittero es liberada.
 * El twittero debe estar registrado en la base de datos.
 * El twittero es eliminado de todas los tweets en los que es mencionado.
 *
 * Entrada:
 *    La base de datos y el twittero a eliminar.
 *
 * Salida:
 *    Tras la llamada el twittero es liberado y se devuelve el numero de tweets
 *    en los que estaba mencionado. Si no estaba mencionado pero el twittero se ha
 *    eliminado correctamente de la base de datos se devuelve 0.
 *    Si el twittero no existe en la base de datos se devuelve -1.
 */
int data_base_remove_twittero(data_base *db, twittero *tt);

/**
 * Función para volcar el contenido de la base de datos por pantalla
 * 
 * Entrada:
 *    El puntero a la base de datos
 * Salida:
 *    
 */
void dump_data_base(data_base *db);

#endif
